@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2021 Mike Fährmann
+# Copyright 2021-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -20,6 +20,7 @@ _CACHE = {}
 _CONVERSIONS = None
 _GLOBALS = {
     "_env": lambda: os.environ,
+    "_lit": lambda: _literal,
     "_now": datetime.datetime.now,
 }
 
@@ -43,6 +44,8 @@ def parse(format_string, default=None):
             cls = ExpressionFormatter
         elif kind == "M":
             cls = ModuleFormatter
+        elif kind == "F":
+            cls = FStringFormatter
 
     formatter = _CACHE[key] = cls(format_string, default)
     return formatter
@@ -206,10 +209,21 @@ class ModuleFormatter():
         self.format_map = getattr(module, function_name)
 
 
+class FStringFormatter():
+    """Generate text by evaluaring an f-string literal"""
+
+    def __init__(self, fstring, default=None):
+        self.format_map = util.compile_expression("f'''" + fstring + "'''")
+
+
 def parse_field_name(field_name):
     first, rest = _string.formatter_field_name_split(field_name)
     funcs = []
 
+    if first[0] == "'":
+        funcs.append(operator.itemgetter(first[1:-1]))
+        first = "_lit"
+
     for is_attr, key in rest:
         if is_attr:
             func = operator.attrgetter
@@ -245,7 +259,7 @@ def parse_format_spec(format_spec, conversion):
             "C": string.capwords,
             "j": json.dumps,
             "t": str.strip,
-            "T": util.to_timestamp,
+            "T": util.datetime_to_timestamp_string,
             "d": text.parse_timestamp,
             "U": text.unescape,
             "S": util.to_string,
@@ -335,3 +349,15 @@ def _default_format(format_spec):
     def wrap(obj):
         return format(obj, format_spec)
     return wrap
+
+
+class Literal():
+    # __getattr__, __getattribute__, and __class_getitem__
+    # are all slower than regular __getitem__
+
+    @staticmethod
+    def __getitem__(key):
+        return key
+
+
+_literal = Literal()
