@@ -39,8 +39,9 @@ class AppendCommandAction(argparse.Action):
 class DeprecatedConfigConstAction(argparse.Action):
     """Set argparse const values as config values + deprecation warning"""
     def __call__(self, parser, namespace, values, option_string=None):
-        print("warning: {} is deprecated. Use {} instead.".format(
-            "/".join(self.option_strings), self.choices), file=sys.stderr)
+        sys.stderr.write(
+            "warning: {} is deprecated. Use {} instead.\n".format(
+                "/".join(self.option_strings), self.choices))
         namespace.options.append(((), self.dest, self.const))
 
 
@@ -59,7 +60,7 @@ class ParseAction(argparse.Action):
 class Formatter(argparse.HelpFormatter):
     """Custom HelpFormatter class to customize help output"""
     def __init__(self, *args, **kwargs):
-        super().__init__(max_help_position=50, *args, **kwargs)
+        super().__init__(max_help_position=30, *args, **kwargs)
 
     def _format_action_invocation(self, action):
         opts = action.option_strings[:]
@@ -113,11 +114,6 @@ def build_parser():
         help=("Filename format string for downloaded files "
               "('/O' for \"original\" filenames)"),
     )
-    general.add_argument(
-        "--cookies",
-        dest="cookies", metavar="FILE", action=ConfigAction,
-        help="File to load additional cookies from",
-    )
     general.add_argument(
         "--proxy",
         dest="proxy", metavar="URL", action=ConfigAction,
@@ -134,6 +130,18 @@ def build_parser():
         help="Delete cached login sessions, cookies, etc. for MODULE "
              "(ALL to delete everything)",
     )
+    general.add_argument(
+        "--cookies",
+        dest="cookies", metavar="FILE", action=ConfigAction,
+        help="File to load additional cookies from",
+    )
+    general.add_argument(
+        "--cookies-from_browser",
+        dest="cookies_from_browser", metavar="BROWSER[+KEYRING][:PROFILE]",
+        help=("Name of the browser to load cookies from, "
+              "with optional keyring name prefixed with '+' and "
+              "profile prefixed with ':'"),
+    )
 
     output = parser.add_argument_group("Output Options")
     output.add_argument(
@@ -400,6 +408,18 @@ def build_parser():
         },
         help="Convert Pixiv Ugoira to WebM in VP9 lossless mode",
     )
+    postprocessor.add_argument(
+        "--ugoira-conv-copy",
+        dest="postprocessors", action="append_const", const={
+            "name"             : "ugoira",
+            "extension"        : "mkv",
+            "ffmpeg-args"      : ("-c:v", "copy"),
+            "ffmpeg-twopass"   : False,
+            "repeat-last-frame": False,
+            "whitelist"        : ("pixiv", "danbooru"),
+        },
+        help="Convert Pixiv Ugoira to MKV without re-encoding any frames",
+    )
     postprocessor.add_argument(
         "--write-metadata",
         dest="postprocessors",
